import inspect
import json
import os.path

import httpretty

import ckanapi_exporter.exporter as exporter


def _absolute_path(relative_path):
    """Return an absolute path given a path relative to this Python file."""
    return os.path.join(os.path.dirname(os.path.abspath(
        inspect.getfile(inspect.currentframe()))), relative_path)


def _mock_package_search(url, body):
    """Mock the package_search API of the given CKAN site.

    Make it return the given body dict as JSON.

    """
    if not url.endswith("/"):
        url = url + "/"
    # This is a bit brittle, we're relying on the fact that ckanapi will end
    # up posting to this URL.
    url = url + "api/action/package_search"
    body = json.dumps(body)
    httpretty.register_uri(httpretty.POST, url, body=body,
                           content_type="application/json")


@httpretty.activate
def test_export():

    # The CKAN site whose API we're going to mock.
    url = "http://demo.ckan.org"

    # The mock package_search result from CKAN.
    body = {
        "help": "blah blah blah",
        "success": True,
        "result": {
            "count": 3,
            "sort":"score desc, metadata_modified desc",
            "facets": {},
            "results": [
                {
                    "author": "Guybrush Threepwood",
                    "notes": "test notes 1",
                    "resources": [
                        {"format": "CSV"},
                        {"format": "JSON"},
                    ],
                    "extras": [
                        {"key": "Update Frequency",
                         "value": "yearly",
                        },
                    ],
                },
                {
                    "author": "Elaine Marley",
                    "notes": "test notes 2",
                    "resources": [
                        {"format": "PDF"},
                        {"format": "TXT"},
                    ],
                    "extras": [
                        {"key": "update",
                         "value": "monthly",
                        },
                    ],
                },
                {
                    "author": "Mancomb Seepgood",
                    "notes": "test notes 3",
                    "resources": [
                        {"format": "XLS"},
                        {"format": "JPEG"},
                    ],
                    "extras": [
                        {"key": "Updated",
                         "value": "weekly",
                        },
                    ],
                },
            ]
        }
    }
    _mock_package_search(url, body)

    csv_string = exporter.export(url, _absolute_path("test_columns.json"))

    assert csv_string == (
        "Data Owner,Description,Formats,Update Frequency\r\n"
        'Guybrush Threepwood,test notes 1,"CSV, JSON",yearly\r\n'
        'Elaine Marley,test notes 2,"PDF, TXT",monthly\r\n'
        'Mancomb Seepgood,test notes 3,"XLS, JPEG",weekly\r\n'
    )



import sys
import argparse
import ast

import ckanapi
import losser.losser
import losser.cli
import math


VERSION = '0.1.0'

def get_datasets_from_ckan(url, apikey, params=None):
    """Gets the datasets from CKAN API and passed in parameters

    Parameters
    ----------
    url: str
        The url of the CKAN instance.
    apikey: str, optional
        The CKAN api key to access private datasets.
    params: str, optional
        A literal string representing a dictionary of parameters to pass to CKAN.

    Returns
    -------
    array:
        All the datasets requested in an array.

    """
    user_agent = ('ckanapi-exporter/{version} '
                  '(+https://github.com/ckan/ckanapi-exporter)').format(
                          version=VERSION)
    api = ckanapi.RemoteCKAN(url, apikey=apikey, user_agent=user_agent)

    #find out if there are more than the hard coded 1000 datasets and how many pages
    arguments = {'rows': 1000, 'start': 0}
    if params is not None:
        params_dict = ast.literal_eval(params)
        arguments.update(params_dict)

    response = api.call_action('package_search', arguments)
    num_pages = int(math.ceil(response['count']/1000.0))

    #loop over to collect up all datasets from the CKAN instance
    datasets = []
    for page in range(0, num_pages):
        arguments['start'] = page * 1000
        paged_response = api.call_action('package_search', arguments)
        #merge these results into one dictionary
        datasets.extend(paged_response["results"])

    #return the large array to go about it's business
    return datasets


def extras_to_dicts(datasets):
    for dataset in datasets:
        extras_dict = {}
        for extra in dataset.get("extras", []):
            key = extra["key"]
            value = extra["value"]
            assert key not in extras_dict
            extras_dict[key] = value
        dataset["extras"] = extras_dict


def export(url, columns, apikey=None, params=None, pretty=False):
    datasets = get_datasets_from_ckan(url, apikey, params)
    extras_to_dicts(datasets)
    csv_string = losser.losser.table(datasets, columns, csv=True,
                                     pretty=pretty)
    return csv_string


def main(args=None):
    parent_parser = losser.cli.make_parser(
        add_help=False, exclude_args=["-i"])
    parser = argparse.ArgumentParser(
        description="Export datasets from a CKAN site to JSON or CSV.",
        parents=[parent_parser],
    )
    parser.add_argument(
        "--url",
        help="the root URL of the CKAN site to export datasets from, "
             "for example: 'http://demo.ckan.org'",
        required=True,
    )
    parser.add_argument(
        "--apikey",
        help="the API key to use when fetching datasets from the CKAN site, "
             "use this option if you want to export private datasets as well "
             "as public ones",
        )
    parser.add_argument(
        "--params",
        help="a dictionary of CKAN API parameters passed into the export query",
        )
    try:
        parsed_args = losser.cli.parse(parser=parser)
    except losser.cli.CommandLineExit as err:
        sys.exit(err.code)
    except losser.cli.CommandLineError as err:
        if err.message:
            parser.error(err.message)
    csv_string = export(
        parsed_args.url, parsed_args.columns, parsed_args.apikey,
        parsed_args.params, pretty=parsed_args.pretty)
    sys.stdout.write(csv_string)


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.markdown'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ckanapi-exporter',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # http://packaging.python.org/en/latest/tutorial.html#version
    version='0.1.0',

    description='Export dataset metadata from CKAN to Excel',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/ckan/ckanapi-exporter',

    # Author details
    author='Sean Hammond',
    author_email='ckanapi-exporter@seanh.cc',

    # Choose your license
    license='AGPL',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
    ],

    # What does your project relate to?
    keywords='ckan csv export excel api',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when your
    # project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/technical.html#install-requires-vs-requirements-files
    install_requires=['ckanapi', 'losser'],

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages.
    # see http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'ckanapi-exporter=ckanapi_exporter.exporter:main',
        ],
    },
)
